(function () {

    'use strict';

    var isWeChatGame = !!window["wx"];

    function boot (gameName) {
        if(cc.native) {
            cc.native.writeLog("boot game");
        }

        var gamePath = gameName + "/";
        // require settings.js
        require(gamePath + 'src/settings.js');

        var settings = window._CCSettings;
        window._CCSettings = undefined;

        if ( !settings.debug ) {
            var uuids = settings.uuids;

            var rawAssets = settings.rawAssets;
            var assetTypes = settings.assetTypes;
            var realRawAssets = settings.rawAssets = {};
            for (var mount in rawAssets) {
                var entries = rawAssets[mount];
                var realEntries = realRawAssets[mount] = {};
                for (var id in entries) {
                    var entry = entries[id];
                    var type = entry[1];
                    // retrieve minified raw asset
                    if (typeof type === 'number') {
                        entry[1] = assetTypes[type];
                    }
                    // retrieve uuid
                    realEntries[uuids[id] || id] = entry;
                }
            }

            var scenes = settings.scenes;
            for (var i = 0; i < scenes.length; ++i) {
                var scene = scenes[i];
                if (typeof scene.uuid === 'number') {
                    scene.uuid = uuids[scene.uuid];
                }
            }

            var packedAssets = settings.packedAssets;
            for (var packId in packedAssets) {
                var packedIds = packedAssets[packId];
                for (var j = 0; j < packedIds.length; ++j) {
                    if (typeof packedIds[j] === 'number') {
                        packedIds[j] = uuids[packedIds[j]];
                    }
                }
            }
        }

        var onStart = function () {
            if(cc.native) {
                cc.native.writeLog("onStart");
            }
            cc.view.resizeWithBrowserSize(true);

            if (!isWeChatGame) {
                // UC browser on many android devices have performance issue with retina display
                if (cc.sys.os !== cc.sys.OS_ANDROID || cc.sys.browserType !== cc.sys.BROWSER_TYPE_UC) {
                    cc.view.enableRetina(true);
                }
                if (cc.sys.isBrowser) {
                    setLoadingDisplay();
                }

                if (cc.sys.isMobile) {
                    if (settings.orientation === 'landscape') {
                        cc.view.setOrientation(cc.macro.ORIENTATION_LANDSCAPE);
                    }
                    else if (settings.orientation === 'portrait') {
                        cc.view.setOrientation(cc.macro.ORIENTATION_PORTRAIT);
                    }
                    // qq, wechat, baidu
                    cc.view.enableAutoFullScreen(
                        cc.sys.browserType !== cc.sys.BROWSER_TYPE_BAIDU &&
                        cc.sys.browserType !== cc.sys.BROWSER_TYPE_WECHAT &&
                        cc.sys.browserType !== cc.sys.BROWSER_TYPE_MOBILE_QQ
                    );
                }
                
                // Limit downloading max concurrent task to 2,
                // more tasks simultaneously may cause performance draw back on some android system / brwosers.
                // You can adjust the number based on your own test result, you have to set it before any loading process to take effect.
                if (cc.sys.isBrowser && cc.sys.os === cc.sys.OS_ANDROID) {
                    cc.macro.DOWNLOAD_MAX_CONCURRENT = 2;
                }
            }

            // init assets
            cc.AssetLibrary.init({
                libraryPath: gamePath + 'res/import',
                rawAssetsBase: gamePath + 'res/raw-',
                rawAssets: settings.rawAssets,
                packedAssets: settings.packedAssets,
                md5AssetsMap: settings.md5AssetsMap
            });

            var launchScene = settings.launchScene;

            // load scene
            cc.director.loadScene(launchScene, null,
                function () {
                    cc.loader.onProgress = null;
                    console.log('Success to load scene: ' + launchScene);
                }
            );
        };

        // require project.js
        console.log('require project.js');
        require(gamePath + 'src/project.js')

        console.log('init search path...');
        jsb.fileUtils.setSearchPaths([gamePath]);

        // set jsList
        var jsList = settings.jsList;
        var bundledScript = settings.debug ? 'src/project.dev.js' : 'src/project.js';
        if (jsList) {
//            jsList = jsList.map(function (x) { return 'src/' + x; });
            jsList.push(bundledScript);
        }
        else {
            jsList = [bundledScript];
        }


        var option = {
            //width: width,
            //height: height,
            id: 'GameCanvas',
            scenes: settings.scenes,
            debugMode: settings.debug ? cc.DebugMode.INFO : cc.DebugMode.ERROR,
            showFPS: isWeChatGame ? false : settings.debug,
            frameRate: 60,
            jsList: jsList,
            groupList: settings.groupList,
            collisionMatrix: settings.collisionMatrix,
            renderMode: 1
        };

        cc.game.run(option, onStart);
    }

    console.log('require jsb_polyfill.js');
    require('src/jsb_polyfill.js');

    console.log('require native.js');
    require('src/native.js');
    console.log('requrie native end');

//    if(cc.sys.os === cc.sys.OS_ANDROID) {
//        // Android
//        // notify java code to do things
//        cc.director.notificationNative = function (eventName, data) {
//            jsb.reflection.callStaticMethod("com/yx/cocos/CocosManager", "notificationNative", "(Ljava/lang/String;Ljava/lang/String;)V", eventName, data);
//        }
//    }
    cc.director.notificationNative = function (eventName, data) {
        var encodeData = encodeURIComponent(data);
        jsb.reflection.callStaticMethod("com/yx/pkgame/cocos/CocosManager", "notificationNative", "(Ljava/lang/String;Ljava/lang/String;)V", eventName, encodeData);
    }
    // receive start Game
    cc.director.startGame = function(data) {
//        var data = JSON.parse(decodeURIComponent(data));
        if(cc.native) {
            cc.native.writeLog("startGame" + data)
        }
        boot(data);
    }

    // notify java cocos loaded
    if(cc.native) {
        cc.native.writeLog("cocos loaded");
    }
    if(cc.sys.os === cc.sys.OS_ANDROID && cc.director.notificationNative) {
        cc.director.notificationNative("COCOS_ANDROID_LOADED", "{}");
    }
})();
